import {useEffect, useState, useRef, useCallback} from 'react';
import mapboxgl from 'mapbox-gl';
import {v4} from 'uuid';
import {Subject} from 'rxjs';
// eslint-disable-next-line import/no-webpack-loader-syntax
mapboxgl.workerClass = require("worker-loader!mapbox-gl/dist/mapbox-gl-csp-worker").default;

mapboxgl.accessToken = 'pk.eyJ1IjoibXJib21iYWRpbCIsImEiOiJja3UxMWw5NmUweW1zMm5wYzZ5OXpjaDkwIn0.jOrA50JvuX2aeuccMedwzw';

export const useMapbox = (initialValues) => {
    const mapDiv = useRef();
    const setRef = useCallback(
        (node) => {
            mapDiv.current = node;
        },
        [],
    )
    
    // Mapa y coordenadas
    const mapa = useRef();
    const [coords, setCoords] = useState(initialValues);

    // Repositorio de markersRepo que se irán actualizando 
    // durante conexiones a sockets
    const markersRepo = useRef({});
    const stopsRepo = useRef({});

    const addStop = useCallback(parada => {
        const {longitude, latitude} = parada;

        const el = document.createElement('div');
        el.style.width = `16px`;
        el.style.height = `16px`;
        el.style.backgroundSize = '100%';
        el.style.backgroundImage = "url(/images/bus_stop.png)";
        
        const marker = new mapboxgl.Marker(el);
        
        marker.id = v4();
        marker
            .setLngLat([longitude, latitude])
            .setPopup(
                new mapboxgl.Popup({ offset: 25 }) // add popups
                            .setHTML(`<p>Nº de Parada ${parada.codeStop}</p><p>${parada.name}</p>`)
                )
            .addTo(mapa.current)
        // Nuevo marcador al repositorio de markersRepo
        stopsRepo.current[marker.id] = marker;
    }, [])

    // Borra todas las paradas del mapa
    const deleteAllBusStops = useCallback( e => {
        for (let markerStored in stopsRepo.current){
            stopsRepo.current[markerStored].remove();                
        }
    }, [])

    // Repositorio de markersRepo que se irán actualizando 
    // durante conexiones a sockets
    const busMarkersRepo = useRef({});
    // Observadores
    //const busMarkerMovement = useRef(new Subject());
    const newBusMarker = useRef(new Subject());


    const addBusMarker = useCallback((e, id, line='') => {
        const {lng, lat} = e.lngLat || e;
        const el = document.createElement('div');
        el.style.width = `24px`;
        el.style.height = `24px`;
        el.style.backgroundSize = '100%';
        el.style.backgroundImage = "url(/images/bus_icon.png)";
        el.className = 'busIcon';
  

        const busMarker = new mapboxgl.Marker(el);
        busMarker.id = id ?? v4(); // TODO: Si marcador ya tiene ID, hacer otra cosa.
        busMarker
            .setLngLat([lng, lat])
            .setPopup(
                new mapboxgl.Popup({ offset: 25 }) // add popups
                    .setHTML(`<p><strong>Nº de línea:</strong> ${line}</p>
                            <p><strong>Identificador</strong>: ${id}</p>`)
                )
            .addTo(mapa.current)
            //.setDraggable(true);            
        // Nuevo marcador al repositorio de markersRepo
        busMarkersRepo.current[busMarker.id] = busMarker;

        // Observamos nuevo marcador
        !id && newBusMarker.current.next({ id: busMarker.id, lng, lat});

    }, [],)
    
    const moveBusMarker = useCallback((busMarker) => {
        busMarkersRepo.current[busMarker.id]?.setLngLat([busMarker.lng, busMarker.lat]);
     }, [])

    const removeBusMarker = useCallback((identification) => {        
        busMarkersRepo.current[identification]?.remove();
    }, [])

    // Añadir rutas al mapa
    // durante conexiones a sockets
    const newRoute = useRef(new Subject());
    const routesRepo = useRef([]);
    const [avisaReact, setAvisaReact] = useState(false)
    const [renderiza, setRenderiza] = useState([]);

    const addBusRoute = useCallback((ruta, color, id = '') => {            

        const loadSource = ()=> {
                routesRepo.current.forEach(e =>{
                    if(e.includes(id)){
                        if(mapa.current.getLayer(e)){
                            mapa.current.removeLayer(e)
                            mapa.current.removeSource(e)
                        }
                    }
                })

                routesRepo.current = routesRepo.current.filter(e => !e.includes(id));
                setRenderiza(routesRepo.current.filter(e => !e.includes(id)));


                if(ruta?.length){
                    const routeId = id + "%" + v4()|| v4();                    
                    routesRepo.current.push(routeId);
                    setRenderiza(prevCount => prevCount.concat(routeId))
                    mapa.current?.addSource(routeId, {
                        'type': 'geojson',
                        'data': {
                            'type': 'Feature',
                            'properties': {},
                            'geometry': {
                                'type': 'LineString',
                                'coordinates': ruta
                            }
                        }
                    })
                    .addLayer({
                        'id': routeId,
                        'type': 'line',
                        'source': routeId,
                        'layout': {
                            'line-join': 'round',
                            'line-cap': 'round'
                        },
                        'paint': {
                            'line-color': color,
                            'line-width': 5
                        }
                    })
                    // Observamos nueva ruta
                    newRoute.current.next({ id: routeId})
                    
                }
                mapa.current.off('data', loadSource);                
                setAvisaReact(true)
        }
        // Dejamos un listener para esperar a que carguen los estilos
        mapa.current.on('data', loadSource);     
        setAvisaReact(true)
    },[])

      // Valores iniciales del mapa
    useEffect(() => {
        const map = new mapboxgl.Map({
            container: mapDiv.current,
            style: 'mapbox://styles/mapbox/streets-v11',
            center: [ initialValues.lng, initialValues.lat ],
            zoom: initialValues.zoom
        });

        mapa.current = map;
    }, [initialValues])

    // Cuando se mueve el mapa
    useEffect(() => {
        mapa.current?.on('move', () => {
            const {lng, lat} = mapa.current.getCenter();
            setCoords({
                lng: lng.toFixed(4),
                lat: lat.toFixed(4),
                zoom: mapa.current.getZoom().toFixed(2)
            })
        })
        
    }, [])

    const changeMapFocus = useCallback((lng, lat) => {
        setTimeout(() => {
            mapa.current.flyTo({                
                zoom: 13
            });
            setTimeout(() => {
                mapa.current.flyTo({                           
                    center: [lng.toFixed(4), lat.toFixed(4)],
                });
                setTimeout(() => {
                    mapa.current.flyTo({
                        zoom: 17
                    });
                }, 1500);  
            }, 1000);                                    
         }, 500); 
    },[]);
    
    return {        
        coords,
        setRef,
        markersRepo$: markersRepo.current,
        newBusMarker$: newBusMarker.current,
        addBusMarker,
        addStop,
        addBusRoute,
        routesRepo$: routesRepo.current,
        mapa$: mapa.current,
        deleteAllBusStops,
        avisaReact,
        setAvisaReact,
        renderiza,
        setRenderiza,
        changeMapFocus,
        moveBusMarker,
        removeBusMarker
    }
}
