const { Bus } = require('../model/Bus');
const {Line} = require('../model/Line');
const {Route} = require('../model/Route');
const {stop} = require('../model/stop');
const User = require('../model/User');

exports.readLine = async (req, res) =>{    
    try {
        const {codeLine} = req.params;
        const lines = await Line.findOne({codeLine: codeLine.toString()}).populate([
            {
                path: 'stops',            
                populate: [
                    {
                        path: 'details.line'
                    },
                    {
                        path: 'lines'
                    },            
                ]
            },
            {
                path: 'route'
            },
            {
                path: 'firstStop',
                populate: [
                    {
                        path: 'details.line'
                    }
                ]
            },
            {
                path: 'lastStop',
                populate: [
                    {
                        path: 'details.line'
                    }
                ]
            }
            
          ])
        if(!lines){
            res.status(404).json({
                ok: false,
                msg: "No existe la línea"
            });
        }
        else{
            res.status(200).json({
                ok: true,
                data: lines
            });
        }

        console.log('1 línea devuelta');
        //console.log(lines);
    } catch (error) {
        res.status(500).json({error: error.message});
    }
}

exports.readAllLines = async (req, res) => {
    try{
        const lines = await Line.find({}).sort( { codeLine: 1 } ).populate('route')

        if(!lines){
            res.status(404).json({
                ok: false,
                msg: "No existe la línea"
            });
        }
        else{
            res.status(200).json({
                ok: true,
                data: lines
            });
        }

        console.log('Líneas devueltas');
        
    }catch (error){
        res.status(500).json({error: error.message});
    }
}

exports.createLine = async (req, res) =>{    
    const line = new Line(req.body);
    try {        

        //Buscar cada línea asociada en el array lines
        // e incluir la parada
        let lineExists = await Line.findOne({codeLine: line.codeLine})        
        if(lineExists){
            res.status(401).json({
                ok: false,
                msg: 'Ya existe esa línea'
            });
        }
        else{
            const lineaGuardada = await line.save();            

            // Hacemos un barrido de stops guardados y los alocamos en 
            // el campo lines de cada uno de ellos.
            line.stops.forEach(async(stopOfLine) => {                
                try{
                    const stopFound = await stop.findOne({_id: stopOfLine});
                    await stopFound.lines.addToSet(lineaGuardada);
                    await stopFound.save();    
                }
                catch (error){
                    console.log(error)
                    return  res.status(500).json({
                        ok: false,
                        msg: 'Hable con el administrador'
                    });
                }
            })

            res.status(201).json({
                ok: true,
                data: lineaGuardada
            })
        }

    } catch (error) {
        console.log(error)
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador'
        });
    }
}


exports.editLine = async (req, res) =>{ 

    // Con la edición, tenemos un problema.
    // Cuando nos cargamos una parada, tenemos que borrarlas del stop.lines que sea
    // El modus operandi es el siguiente:
    // 1 - Recibo línea
    // 2 - Compruebo las paradas que tiene actualmente
    // 3 - Comparo con las que estoy mandando
    // 4 - Si hay diferencias, elimina las que no se incluyan en los nuevos datos.
    // const arrayQueEnvio = ['1', '2']
    // const arrayQueTengo = ['1', '2', '3', '4']    
    // arrayQueTengo.filter(x => !arrayQueEnvio.includes(x));
    // ['3', '4'] <-- resultados a borrar
    // 7 - El resultado, si es que sale algo, se recorre y se elimina con un forEach de cada elemento y pull en stop.lines

    const lineId = req.params.id;
    try {

        let lineExists = await Line.findOne({_id : lineId})        
        if(!lineExists){
            res.status(400).json({
                ok: false,
                msg: 'No existe esa línea'
            });
        }
        else{
            const newLine = {
                ...req.body
            }
    
            // Configuramos la nueva línea
            const newLineUpdated = await Line.findByIdAndUpdate( lineId, newLine, { new: true } );

            // Buscamos las paradas a eliminar
            const paradasAEliminar = lineExists.stops.filter(x => !newLineUpdated.stops.includes(x));
                      
            // Quitamos las que ya no están 
            paradasAEliminar.forEach(async(parada) =>{
                let paradita = await stop.findById(parada.toString());
                await paradita.lines.pull(newLineUpdated);
                await paradita.save();
            })

            // Ponemos las nuevas o dejamos las que estaban
            newLineUpdated.stops.forEach(async (parada) =>{
                try {
                    let paradita = await stop.findById(parada._id);
                    await paradita.lines.addToSet(newLineUpdated);
                    await paradita.save();    
                } catch (error) {
                    console.log(error)                
                }
            })

            res.status(201).json({
                ok: true,
                data: newLineUpdated
            })
        }

    } catch (error) {        
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador: ' + error.message
        });
    }
}

exports.deleteLine = async (req, res) =>{ 
    const lineId = req.params.id;
    try {

        let lineExists = await Line.findOne({_id : lineId})        
        if(!lineExists){
            res.status(400).json({
                ok: false,
                msg: 'No existe esa línea'
            });
        }
        else{
            const newLine = {
                ...req.body
            }
            
            // Buscamos las paradas de donde salirse
            const paradasDeDondeSalirse = lineExists.stops;

            // Quitamos las que ya no están en paradas
            paradasDeDondeSalirse.forEach(async(_parada) =>{
                let parada = await stop.findById(_parada._id);
                await parada.lines.pull(lineExists);
                console.log("Nos cargamos línea de parada: ", parada.name)
                await parada.save();
            })
            
            // Quitar de favoritos en users
            const usuarios = await User.find( { favoriteLines: {$all: [lineExists]}} );            
            usuarios.forEach(async _usuario =>{
                let usuario = await User.findById(_usuario.id)
                await usuario.favoriteLines.pull(lineExists);
                console.log("Nos cargamos línea de usuario: ", usuario.name)
                await usuario.save();
            })

            
            // Quitar de rutas
            await Route.updateMany(
                {},
                { $pullAll: { lines: [lineExists._id] } },
                { multi: true }
            );

            //Borrar autobuses de la línea
            await Bus.remove({line: lineExists._id });

            
            // Borramos la línea
            await Line.findByIdAndDelete( lineId );
            res.status(200).json({ ok: true });           

        }

    } catch (error) {
        console.log(error)
        res.status(409).json({
            ok: false,
            msg: 'Hable con el administrador: ' + error.message
        });
    }
}